@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2020 ZNC, see the NOTICE file for details.
+ * Copyright (C) 2004-2024 ZNC, see the NOTICE file for details.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -632,7 +632,7 @@ class CWatcherMod : public CModule {
     
         CString sHostMask = sLine.Token(1);
         CString sTarget = sLine.Token(2);
-        CString sPattern = sLine.Token(3);
+        CString sPattern = sLine.Token(3, true);
 
         CString sMessage;
 
