@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2020 ZNC, see the NOTICE file for details.
+ * Copyright (C) 2004-2024 ZNC, see the NOTICE file for details.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@
 #include "znctest.h"
 
 using testing::HasSubstr;
+using testing::Not;
 
 namespace znc_inttest {
 namespace {
@@ -54,6 +55,50 @@ TEST_F(ZNCTest, NotifyConnectModule) {
         "NOTICE nick :*** user@identifier detached from 127.0.0.1");
 }
 
+TEST_F(ZNCTest, ClientNotifyModule) {
+    auto znc = Run();
+    auto ircd = ConnectIRCd();
+    auto client = LoginClient();
+    client.Write("znc loadmod clientnotify");
+    client.ReadUntil("Loaded module");
+
+    auto check_not_sent = [](Socket& client, std::string wrongAnswer){
+        auto result = QString{client.ReadRemainder()}.toStdString();
+        EXPECT_THAT(result, Not(HasSubstr((wrongAnswer)))) << "Got an answer from the ClientNotifyModule even though we didnt want one with the given configuration";
+    };
+
+    auto client2 = LoginClient();
+    client.ReadUntil(":Another client (127.0.0.1) authenticated as your user. Use the 'ListClients' command to see all 2 clients.");
+    auto client3 = LoginClient();
+    client.ReadUntil(":Another client (127.0.0.1) authenticated as your user. Use the 'ListClients' command to see all 3 clients.");
+
+    // disable notifications for every message
+    client.Write("PRIVMSG *clientnotify :NewOnly on");
+
+    // check that we do not ge a notification after connecting from a know ip
+    auto client4 = LoginClient();
+    check_not_sent(client, ":Another client (127.0.0.1) authenticated as your user. Use the 'ListClients' command to see all 4 clients.");
+
+    // choose to notify only on new client ids
+    client.Write("PRIVMSG *clientnotify :NotifyOnNewID on");
+
+    auto client5 = LoginClient("identifier123");
+    client.ReadUntil(":Another client (127.0.0.1 / identifier123) authenticated as your user. Use the 'ListClients' command to see all 5 clients.");
+    auto client6 = LoginClient("identifier123");
+    check_not_sent(client, ":Another client (127.0.0.1 / identifier123) authenticated as your user. Use the 'ListClients' command to see all 6 clients.");
+
+    auto client7 = LoginClient("not_identifier123");
+    client.ReadUntil(":Another client (127.0.0.1 / not_identifier123) authenticated as your user. Use the 'ListClients' command to see all 7 clients.");
+
+    // choose to notify from both clientids and new IPs
+    client.Write("PRIVMSG *clientnotify :NotifyOnNewIP on");
+
+    auto client8 = LoginClient();
+    check_not_sent(client, ":Another client (127.0.0.1 / identifier123) authenticated as your user. Use the 'ListClients' command to see all 8 clients.");
+    auto client9 = LoginClient("definitely_not_identifier123");
+    client.ReadUntil(":Another client (127.0.0.1 / definitely_not_identifier123) authenticated as your user. Use the 'ListClients' command to see all 9 clients.");
+}
+
 TEST_F(ZNCTest, ShellModule) {
     auto znc = Run();
     auto ircd = ConnectIRCd();
@@ -78,6 +123,11 @@ TEST_F(ZNCTest, WatchModule) {
     ircd.Write(":n!i@h PRIVMSG #znc :\001ACTION foo\001");
     client.ReadUntil(
         ":$*!watch@znc.in PRIVMSG nick :* CTCP: n [ACTION foo] to [#znc]");
+    client.Write("PRIVMSG *watch :add * *spaces *word1 word2*");
+    client.ReadUntil("Adding entry:");
+    ircd.Write(":n!i@h PRIVMSG #znc :SOMETHING word1 word2 SOMETHING");
+    client.ReadUntil(
+        ":*spaces!watch@znc.in PRIVMSG nick :<n:#znc> SOMETHING word1 word2 SOMETHING");
 }
 
 TEST_F(ZNCTest, ModuleCrypt) {
@@ -184,7 +234,7 @@ TEST_F(ZNCTest, KeepNickModule) {
     ircd.ReadUntil("NICK user");
     ircd.Write(":server 435 user_ user #error :Nope :-P");
     client.ReadUntil(
-        ":*keepnick!znc@znc.in PRIVMSG user_ "
+        ":*keepnick!keepnick@znc.in PRIVMSG user_ "
         ":Unable to obtain nick user: Nope :-P, #error");
 }
 
@@ -268,5 +318,17 @@ INSTANTIATE_TEST_CASE_P(SaslInst, SaslModuleTest,
                                 99,
                                 {SaslModuleTest::Prefix() + "YmJi", "Yg=="}}));
 
+TEST_F(ZNCTest, SaslMechsNotInit) {
+    auto znc = Run();
+    auto ircd = ConnectIRCd();
+    auto client = LoginClient();
+    client.Write("znc loadmod sasl");
+    client.Write("PRIVMSG *sasl :set * *");
+    client.ReadUntil("Password has been set");
+    ircd.Write("AUTHENTICATE +");
+    ircd.Write("PING foo");
+    ircd.ReadUntil("PONG foo");
+}
+
 }  // namespace
 }  // namespace znc_inttest
