@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2020 ZNC, see the NOTICE file for details.
+ * Copyright (C) 2004-2024 ZNC, see the NOTICE file for details.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -31,7 +31,7 @@ TEST_F(ZNCTest, Modperl) {
     client.Write("znc loadmod modperl");
     client.Write("znc loadmod perleval");
     client.Write("PRIVMSG *perleval :2+2");
-    client.ReadUntil(":*perleval!znc@znc.in PRIVMSG nick :Result: 4");
+    client.ReadUntil(":*perleval!perleval@znc.in PRIVMSG nick :Result: 4");
     client.Write("PRIVMSG *perleval :$self->GetUser->GetUsername");
     client.ReadUntil("Result: user");
 }
@@ -48,7 +48,7 @@ TEST_F(ZNCTest, Modpython) {
     client.Write("znc loadmod modpython");
     client.Write("znc loadmod pyeval");
     client.Write("PRIVMSG *pyeval :2+2");
-    client.ReadUntil(":*pyeval!znc@znc.in PRIVMSG nick :4");
+    client.ReadUntil(":*pyeval!pyeval@znc.in PRIVMSG nick :4");
     client.Write("PRIVMSG *pyeval :module.GetUser().GetUsername()");
     client.ReadUntil("nick :'user'");
     ircd.Write(":server 001 nick :Hello");
@@ -60,7 +60,7 @@ TEST_F(ZNCTest, Modpython) {
     client.Write("PRIVMSG *controlpanel :Set ClientEncoding $me Western");
     client.Write("JOIN #a\342");
     client.ReadUntil(
-        ":*controlpanel!znc@znc.in PRIVMSG nick :ClientEncoding = UTF-8");
+        ":*controlpanel!controlpanel@znc.in PRIVMSG nick :ClientEncoding = UTF-8");
     ircd.ReadUntil("JOIN #a\xEF\xBF\xBD");
 }
 
@@ -243,5 +243,124 @@ TEST_F(ZNCTest, ModperlNV) {
     client.ReadUntil(":a b");
 }
 
+TEST_F(ZNCTest, ModpythonPackage) {
+    if (QProcessEnvironment::systemEnvironment().value(
+            "DISABLED_ZNC_PERL_PYTHON_TEST") == "1") {
+        return;
+    }
+    auto znc = Run();
+    znc->CanLeak();
+
+    QDir dir(m_dir.path());
+    ASSERT_TRUE(dir.mkpath("modules"));
+    ASSERT_TRUE(dir.cd("modules"));
+    ASSERT_TRUE(dir.mkpath("packagetest"));
+    InstallModule("packagetest/__init__.py", R"(
+        import znc
+        from .data import value
+
+        class packagetest(znc.Module):
+            def OnModCommand(self, cmd):
+                self.PutModule('value = ' + value)
+    )");
+    InstallModule("packagetest/data.py", "value = 'a'");
+
+    auto ircd = ConnectIRCd();
+    auto client = LoginClient();
+    client.Write("znc loadmod modpython");
+    client.Write("znc loadmod packagetest");
+    client.Write("PRIVMSG *packagetest :foo");
+    client.ReadUntil("value = a");
+    InstallModule("packagetest/data.py", "value = 'b'");
+    client.Write("PRIVMSG *packagetest :foo");
+    client.ReadUntil("value = a");
+    client.Write("znc updatemod packagetest");
+    client.Write("PRIVMSG *packagetest :foo");
+    client.ReadUntil("value = b");
+    // Test if python modules are viewable via *status.
+    // https://github.com/znc/znc/issues/1884
+    client.Write("znc listavailmods");
+    client.ReadUntil(":*status!status@znc.in PRIVMSG nick :\x02 packagetest");
+    client.ReadUntil(":*status!status@znc.in PRIVMSG nick :\x02 pyeval\x0F: Evaluates python code");
+}
+
+TEST_F(ZNCTest, ModpythonModperl) {
+    if (QProcessEnvironment::systemEnvironment().value(
+            "DISABLED_ZNC_PERL_PYTHON_TEST") == "1") {
+        return;
+    }
+    auto znc = Run();
+    znc->CanLeak();
+
+    auto ircd = ConnectIRCd();
+    auto client = LoginClient();
+    // https://github.com/znc/znc/issues/1757
+    client.Write("znc loadmod modpython");
+    client.ReadUntil("Loaded module modpython");
+    client.Write("znc loadmod modperl");
+    client.ReadUntil("Loaded module modperl");
+}
+
+TEST_F(ZNCTest, ModpythonCommand) {
+    if (QProcessEnvironment::systemEnvironment().value(
+            "DISABLED_ZNC_PERL_PYTHON_TEST") == "1") {
+        return;
+    }
+
+    auto znc = Run();
+    znc->CanLeak();
+
+    InstallModule("cmdtest.py", R"(
+        import znc
+
+        class cmdtest(znc.Module):
+            def OnLoad(self, args, message):
+                self.AddHelpCommand()
+                self.AddCommand(testcmd)
+                return True
+
+        class testcmd(znc.Command):
+            command = 'ping'
+            args = cmdtest.t_d('ar')
+            description = cmdtest.t_d('blah')
+
+            def __call__(self, line):
+                self.GetModule().PutModule(line + cmdtest.t_s(' pong'))
+    )");
+
+    auto ircd = ConnectIRCd();
+    auto client = LoginClient();
+    client.Write("znc loadmod modpython");
+    client.Write("znc loadmod cmdtest");
+    client.Write("PRIVMSG *cmdtest :ping or");
+    client.ReadUntil(":*cmdtest!cmdtest@znc.in PRIVMSG nick :ping or pong");
+
+    InstallTranslation("cmdtest", "ru_RU", R"(
+        msgid ""
+        msgstr ""
+        "Content-Type: text/plain; charset=UTF-8\n"
+        "Content-Transfer-Encoding: 8bit\n"
+        "Plural-Forms: nplurals=4; plural=((n%10==1 && n%100!=11) ? 0 : ((n%10 >= 2 "
+        "&& n%10 <=4 && (n%100 < 12 || n%100 > 14)) ? 1 : ((n%10 == 0 || (n%10 >= 5 "
+        "&& n%10 <=9)) || (n%100 >= 11 && n%100 <= 14)) ? 2 : 3));\n"
+        "Language: ru_RU\n"
+
+        msgid "ar"
+        msgstr "аргумент"
+
+        msgid "blah"
+        msgstr "бла"
+
+        msgid " pong"
+        msgstr " понг"
+    )");
+
+    client.Write("PRIVMSG *controlpanel :set language $me ru-RU");
+    client.Write("PRIVMSG *cmdtest :help");
+    client.ReadUntil(":*cmdtest!cmdtest@znc.in PRIVMSG nick :\x02ping аргумент\x0F: бла");
+    client.Write("PRIVMSG *cmdtest :ping");
+    client.ReadUntil(":*cmdtest!cmdtest@znc.in PRIVMSG nick :ping понг");
+}
+
 }  // namespace
 }  // namespace znc_inttest
